<?php
/**
 * SFL Code Box
 *
 * A container of raw code used for display purposes in code box.
 *
 * Code Box Fields:
 * ================
 *
 * Title:       [Use post_title  ] - Title of the code box content
 * Code         [Use post_content] - source code to be displayed.
 */

require_once( 'constants.php' );

include_once( 'container-meta.php' );

// Register Code Container Custom Post Type
add_action('init', 'sfl_create_code_container_type');

/**
 * Create the Code Box Container Custom Post Type.
 *
 * This function is run each time a HTTP Page Request is processed during
 * the INIT action stage.  Ideally, it should only need to run once during
 * the ACTIVATE action stage but the information does not appear to be
 * stored in the database for the actual type and information needed to 
 * register the custom post type are not available during the ACTIVATE
 * action stage.
 */

function sfl_create_code_container_type()
{
	// Assign the names associated with this custom post type.
	$labels = array(
		'name'               => __('Code Containers',
								   SFL_CODE_BOX_TD),
		'singlar_name'       => __('Code Container',
								   SFL_CODE_BOX_TD),
		'add_new'            => __('Add New Code Container',
								   SFL_CODE_BOX_TD),
		'add_new_item'       => __('Add New Code Container',
								   SFL_CODE_BOX_TD),
		'edit_item'          => __('Edit Code Container',
								   SFL_CODE_BOX_TD),
		'new_item'           => __('New Code Container',
								   SFL_CODE_BOX_TD),
		'view_item'          => __('View Code Container',
								   SFL_CODE_BOX_TD),
		'all_items'          => __('All Code Containers',
								   SFL_CODE_BOX_TD),
		'menu_name'          => __('Code Container',
								   SFL_CODE_BOX_TD),
		'name_admin_bar'     => __('Code Container',
								   SFL_CODE_BOX_TD),
		'search_items'       => __('Search Code Container',
								   SFL_CODE_BOX_TD),
		'not_found'          => __('Code Containers not found',
								   SFL_CODE_BOX_TD),
		'not_found_in_trash' => __('No Code Container found in Trash', 
								   SFL_CODE_BOX_TD),
		);

	// List of Taxonomies used by this custom post type
	$taxes = array(	);

	// Assign Arguments needed to create this custom post type.
	$args = array(
		'description'        => __('A Collection of Code Containers',   
								   SFL_CODE_BOX_TD),
		'labels'             => $labels,
		'public'             => true,
		'show_ui'            => true,
		'supports'           => array('title', ),
		'hierarchical'       => false,
		'has_archive'        => false,
        'can_export'         => true,
        'menu_position'      => 22,
		'menu_icon'          => 'dashicons-media-document',
		'taxonomies'         => $taxes,
		'map_meta_cap'       => true,
		'capacity_type'      => SFL_Code_Container::TYPE_NAME,
		'rewrite'            => array( 'slug' => SFL_Code_Container::SLUG ),
		);

	// *** Modify Add/Edit Code Content Form ***

	// Now we have everything assembled to register the Custom Post Type
	register_post_type( SFL_Code_Container::TYPE_NAME, $args );

	// Alter Title Field prompt text for this plugin for adding or editing.
	add_filter('enter_title_here', 'sfl_code_container_edit_title_label');

	// Alter the Edit form to place stuff between the TITLE field
	// and the CONTENT field.
//	add_action('edit_form_after_title', 'sfl_code_container_content_label' );

	// *** Metabox actions and filters ***
    // NOTE: So long as we can use post_content for our code container
	//       we do not need the metabox functions.  They are place holders.

	// Add Metabox to be used to add and edit a Link
	add_action( 'add_meta_boxes', 'sfl_code_container_create_metabox' );

	// Add Action to save data from Metabox when a Link is added or edited
	add_action( 'save_post_' . SFL_Code_Container::TYPE_NAME,
				'sfl_code_container_metabox_save', 10, 3 );

	// All Done
	return;

}   // end of sfl_create_code_container_type()


/**
 * Filter function that changes the label displayed in the TITLE input
 * field for the Code Content Add/Edit Form.
 *
 * @param  $title  The original label to be used.
 *
 * @return Returns our label if this is for the appropriate post type.
 *         Otherwise the original label is returned.
 */
function sfl_code_container_edit_title_label( $title )
{
	$post_type = get_current_screen()->post_type;

	if ( SFL_Code_Container::TYPE_NAME == $post_type )
	{
		// This is the custom post type we are looking for.
		$title = __('Enter Code Container Title here', SFL_CODE_BOX_TD);
	}

	return $title;
}

/**
 * Action function that inserts a <label> tag that will display our
 * label for the content input field if the post type is for a
 * Code Container.
 */
function sfl_code_container_content_label()
{
	$post_type = get_current_screen()->post_type;
	
	if ( SFL_Code_Container::TYPE_NAME == $post_type )
	{
		// This is the correct custom post type

		// Add Label for Content input field

		echo '<label class="screen-reader-text" ' .
			'id="content-prompt-text-sfl" ' .
			'for="content">Source Code</label>';
	}

	// All Done
	return;

}   // end of filter sfl_link_edit_after_title()
