<?php
/**
 * Defines the Shortcodes for this plugin.
 *
 * Shortcodes:
 * ===========
 *
 * [code_box {attributes}] { source code } [/code_box]
 *
 * [code_box] Attributes:
 * ----------------------
 * container="title"     -- [optional] The title of the Code Container
 *                          that will be displayed in the code box.
 *                          The Code Container ID can be used instead of
 *                          the title.  This attribute is ignored if 
 *                          source code is passed to the shortcode as 
 *                          the content of the shortcode.
 *       
 * class="classes"       -- [optional] Additional style classes to add to 
 *                          the tag on each line of source code.
 *
 * inner="classes"       -- [optional] Additional style classes to add to
 *                          the inner wrapper element.
 *
 * inner_id="id string"  -- [optional] The unique ID to assign to the 
 *                          inner wrapper element.
 *                          default: sfl-code-box-counter-#
 *                          # == 1,2,3,... for each code box in page post.
 *
 * outer="classes"       -- [optional] Additional style classes to add to
 *                          the inner wrapper element.
 *
 * outer_id="id string"  -- [optional] The unique ID to assign to the 
 *                          inner wrapper element.
 *                          default: sfl-code-box-counter-#
 *                          # == 1,2,3,... for each code box in page post.
 *
 * source code           -- [optional] This is the content of the shortcode.
 *                          If this contains characters, the container
 *                          attribute is ignored.  If the content is empty
 *                          and no container attribute is set, the code
 *                          box will contain nothing.
 */



// Add Shortcode SFL Code Box
add_shortcode( SFL_CODE_BOX_TAG, 'sfl_shortcode_code_box');

/**
 * Shortcode Function that prepares a section of the content for prepending
 * line numbers.
 *
 * @param $raw_attrs    == Attributes set in the shortcode.
 * @param $old_content  == Content between opening and closing tags of 
 *                         the shortcode.
 * @param $tag          == The shortcode tag.  Ignored for now and is 
 *                         assumed to be SFL_CODE_BOX.
 *
 * @return Returns the content wrapped in <div> and <pre> tags with
 *         line in the content wrapped in its own <span> tag.
 *
 * Syntax:
 * =======
 *
 * [code_box {attributes}]{ source code }[/code_box]
 *
 * Attributes:
 * -----------
 * container="title|ID"  -- [optional] The title of the Code Container
 *                          that will be displayed in the code box.
 *                          The Code Container ID can be used instead of
 *                          the title.  This attribute is ignored if 
 *                          source code is passed to the shortcode as 
 *                          the content of the shortcode.
 *       
 * class="classes"       -- [optional] Additional style classes to add to 
 *                          the tag on each line of source code.
 *
 * inner="classes"       -- [optional] Additional style classes to add to
 *                          the inner wrapper element.
 *
 * inner_id="id string"  -- [optional] The unique ID to assign to the 
 *                          inner wrapper element.
 *                          default: sfl-code-box-counter-#
 *                          # == 1,2,3,... for each code box in page post.
 *
 * outer="classes"       -- [optional] Additional style classes to add to
 *                          the inner wrapper element.
 *
 * outer_id="id string"  -- [optional] The unique ID to assign to the 
 *                          inner wrapper element.
 *                          default: sfl-code-box-counter-#
 *                          # == 1,2,3,... for each code box in page post.
 *
 * Content:
 * --------
 *
 * source code           -- [optional] This is the content of the shortcode.
 *                          If this contains characters, the container
 *                          attribute is ignored.  If the content is empty
 *                          and no container attribute is set, the code
 *                          box will contain nothing.

 * We are assuming that content that is to have line numbers is source
 * code (programming, html, etc.).  We surround the content passed to
 * this shortcode function with its own <div><pre> tags with the appropriate
 * classes and each line of code within its own <span> tag.
 * The preset CSS classes add the line number, the switching to mono font
 * and overflow control.
 */
function sfl_shortcode_code_box( $raw_attrs, $old_content, $tag )
{
	static 	$sfl_code_box_id = 0;
	
	// Define the attributes that are used by this shortcode
	$defaults = array(
		SFL_CODE_BOX_CONTAINER_ATTR  =>  0,
		SFL_CODE_BOX_LINE_ATTR       => '',
		SFL_CODE_BOX_OUTER_ATTR      => '',
		SFL_CODE_BOX_INNER_ATTR      => '',
		SFL_CODE_BOX_INNER_ID_ATTR   => '',
		SFL_CODE_BOX_OUTER_ID_ATTR   => '',
		);
	
	// Merge default with raw attributes (removing any raw attributes
	// that do not have cooresponding attributes in the defaults.
	$attrs = shortcode_atts( $defaults, $raw_attrs );

// DEBUG Start
//	echo '<p>Attributes: ' . print_r( $attrs, true ) . '</p><br/>' . PHP_EOL;
//
//	echo  '<p>Original Content: ' . print_r( $old_content, true ) .
//		'</p><br/>' . PHP_EOL;
//	echo '<p>Original Content is ' .
//		( empty($old_content) ? ' empty' : '' ) .
//		( isset($old_content) ? ' IsSet' : '' ) .
//		'</p><br/>' . PHP_EOL;
// DEBUG End
	
	// If multiple code boxes are displayed on the same post/page
	// We need to be able to uniquely identify both the inner
	// and outer wrappers.
	$sfl_code_box_id++;
	
	// Prepare the tags used for the outer division wrapper
	$outer_id = ( $attrs[SFL_CODE_BOX_OUTER_ID_ATTR] != '' ?
				  $attrs[SFL_CODE_BOX_OUTER_ID_ATTR] :
				  'sfl-code-box-' . $sfl_code_box_id );
	
	$outer_classes = ( $attrs[SFL_CODE_BOX_OUTER_ATTR] != ''?
					   (' ' . $attrs[SFL_CODE_BOX_OUTER]) : '' ) ;
	
	$open_outer = '<div id="' . $outer_id . '" ' .
		'class="' . $outer_classes . '" >' . PHP_EOL;
	
	$close_outer  = '</div>';
	
	// Prepare the tags for the inner division wrapper.
	$inner_id = ( $attrs[SFL_CODE_BOX_INNER_ID_ATTR] != '' ?
				  $attrs[SFL_CODE_BOX_INNER_ID_ATTR] :
				  'sfl-code-box-article-' . $sfl_code_box_id );
	
	$inner_classes = 'sfl-code-box' .
		( $attrs[SFL_CODE_BOX_INNER_ATTR] != '' ?
		  (' ' . $attrs[SFL_CODE_BOX_INNER_ATTR]) : '' );
	
	$open_inner = '<article id="' . $inner_id . '" ' .
		'class="' . $inner_classes . '" >' . PHP_EOL;
	
	$close_inner = '</article>' . PHP_EOL;
	
	// Note, unlike <div> and <pre> tags, there are no default
	// class values for <span> and no IDs are assigned
	$line_classes = ( $attrs[SFL_CODE_BOX_LINE_ATTR] != '' ?
					  ' class="' . $attrs[SFL_CODE_BOX_LINE_ATTR] .
					  '" ' : '' );
	
	$open_span = '<span' . $line_classes . '>';
	
	$close_span = '</span>'; 
	
	// Determine what the source code is.
	if ( empty( $old_content ) )
	{
		// Get source code from Code Container
		$raw_content = sfl_shortcode_code_box_get_content( $attrs );
	}
	else
	{
		// Modify content to under the work that wpautop() did.
		// 1) Replace all </p> and <p> with PHP_EOL.
		// 2) Replace all <br /> with nothing
	
		$breaks        = array('<br />', '<br/>');
		$new_lines     = array('<p>', '</p>');
	
	
		$raw_content = str_replace( $new_lines, PHP_EOL,
									str_replace( $breaks, '',
												 $old_content ) );

		// Need to add in Break Lines for content from page
		// Why here and not there, I do not know.
		$close_span .= '<br />';
   
	}
	// Make the text safe for HTML
	$new_content = esc_attr( $raw_content );
	
	// Apply box tags
	$content = $open_outer . $open_inner;

	$lines = explode( PHP_EOL, $new_content );
	
	// Apply <span> tags to each line.
	foreach( $lines as $line )
	{
		$content .=  $open_span . $line . $close_span . PHP_EOL;
	}
	
	// Apply box closing tags
	$content .= $close_inner . $close_outer;
	
	return $content;
	
}   // end of sfl_shortcode_code_box()


/**
 *
 */
function sfl_shortcode_code_box_get_content( $attrs )
{
	// Determine if the Container Attribute is set to an integer (ID)
	// or a string.
	$container = $attrs[SFL_CODE_BOX_CONTAINER_ATTR];

	if ( is_numeric( $container) )
	{
		$using_id = true;
	}
	elseif ( is_string( $container ) )
	{
		$using_id = false;
	}
	else
	{
		// Error:  Unknown data type.  
		return 'ERROR: ' . SFL_CODE_BOX_CONTAINER_ATTR . 
			' must be a integer or a string.  Value = ' .
			print_r( $attrs[SFL_CODE_BOX_CONTAINER_ATTR], true );
	}

	// Build Query Arguments
	$query_args = array('post_type'    => SFL_Code_Container::TYPE_NAME, );
	
	if ( $using_id )
	{
		$query_args['post_id'] = $container;
	}
	else
	{
		$query_args['post_title'] = $container;
	}

	// Run Query
	$query = new WP_Query( $query_args );

	if ( $query->post_count == 0 )
	{
		// Warning: No Container was found.
		return 'Warning: No Code Container was found for ' .
			( $using_id ? 'ID = ' : 'Title = ' ) .
			$container;
	}

	if ( $query->post_count > 1 )
	{
		// Warning: Too many containers were returned.
		return 'Warning: ' . $query->post_count .
			' Code Containers were found instead of 1 for ' .
			( $using_id ? 'ID = ' : 'Title = ' ) .
			$container;
	}

	// We know there is one post.  Lets get it
	$query->the_post();

	// Return the content of the container as the source code
	return $query->post->post_content;

	
}   // end of sfl_shortcode_code_box_get_content()
